From 5c73433f80cace5aefbac5f913e9e69379220632 Mon Sep 17 00:00:00 2001
From: Tomas Vondra <tomas.vondra@postgresql.org>
Date: Thu, 26 Aug 2021 23:01:04 +0200
Subject: [PATCH 2/4] Improve Var op Var estimates with the same Var

When estimating (Var op Var) clauses, we can treat the case with the
same Var on both sides as a special case, and we can provide better
selectivity estimate than for the generic case.

For example for (a = a) clause we know it's (1.0 - nullfrac), because
all non-NULL rows are expected to match. Similarly for (a != a), wich
has selectivity 0.0. And the same reasoning can be applied to inequality
comparisons, like (a < a) etc.

This of course depends on the operators having the usual meaning (e.g.
"=" equality) and not some custom operator with different behavior, but
if they use the same estimator it's reasonable to assume that (we
already do that in various other places).

In principle, such clauses are a bit strange and queries are unlikely to
use them very often. But query generators sometimes do this, and these
checks are quite cheap so it's likely a win.
---
 src/backend/utils/adt/selfuncs.c        | 103 +++++-
 src/test/regress/expected/stats_ext.out | 436 ++++++++++++------------
 2 files changed, 320 insertions(+), 219 deletions(-)

diff --git a/src/backend/utils/adt/selfuncs.c b/src/backend/utils/adt/selfuncs.c
index d35e5605de8..9289e9b8da8 100644
--- a/src/backend/utils/adt/selfuncs.c
+++ b/src/backend/utils/adt/selfuncs.c
@@ -210,7 +210,8 @@ static bool get_actual_variable_endpoint(Relation heapRel,
 										 MemoryContext outercontext,
 										 Datum *endpointDatum);
 static RelOptInfo *find_join_input_rel(PlannerInfo *root, Relids relids);
-
+static bool matching_restriction_variables(PlannerInfo *root, List *args,
+										   int varRelid);
 
 /*
  *		eqsel			- Selectivity of "=" for any data types.
@@ -256,6 +257,31 @@ eqsel_internal(PG_FUNCTION_ARGS, bool negate)
 		}
 	}
 
+	/*
+	 * If it's (variable = variable) with the same variable on both sides, it's
+	 * a special case and we know it's not expected to filter anything, so we
+	 * estimate the selectivity as (1.0 - nullfrac), or 0.0 when it's negated.
+	 */
+	if (matching_restriction_variables(root, args, varRelid))
+	{
+		VariableStatData	vardata;
+		double				null_frac = 0.0;
+
+		examine_variable(root, linitial(args), varRelid, &vardata);
+
+		if (HeapTupleIsValid(vardata.statsTuple))
+		{
+			Form_pg_statistic	stats;
+
+			stats = (Form_pg_statistic) GETSTRUCT(vardata.statsTuple);
+			null_frac = stats->stanullfrac;
+		}
+
+		ReleaseVariableStats(vardata);
+
+		return (negate) ? 0.0 : (1.0 - null_frac);
+	}
+
 	/*
 	 * If expression is not variable = something or something = variable, then
 	 * punt and return a default estimate.
@@ -1408,6 +1434,22 @@ scalarineqsel_wrapper(PG_FUNCTION_ARGS, bool isgt, bool iseq)
 	Oid			consttype;
 	double		selec;
 
+	/*
+	 * Handle (variable < variable) and (variable <= variable) with the same
+	 * variable on both sides as a special case. The strict inequality should
+	 * not match any rows, hence selectivity is 0.0. The other case is about
+	 * the same as equality, so selectivity is 1.0.
+	 */
+	if (matching_restriction_variables(root, args, varRelid))
+	{
+		/* The case with equality matches all rows, so estimate it as 1.0. */
+		if (iseq)
+			PG_RETURN_FLOAT8(1.0);
+
+		/* Strict inequality matches nothing, so selectivity is 0.0. */
+		PG_RETURN_FLOAT8(0.0);
+	}
+
 	/*
 	 * If expression is not variable op something or something op variable,
 	 * then punt and return a default estimate.
@@ -4906,6 +4948,65 @@ get_restriction_variable(PlannerInfo *root, List *args, int varRelid,
 	return false;
 }
 
+/*
+ * matching_restriction_variable
+ *		Check if the two arguments of a restriction clause refer to the same
+ *		variable, i.e. if the condition is of the form (variable op variable).
+ *		We can deduce selectivity for such (in)equality clauses.
+ *
+ * Inputs:
+ *	root: the planner info
+ *	args: clause argument list
+ *	varRelid: see specs for restriction selectivity functions
+ *
+ * Returns true if the same variable is on both sides, otherwise false.
+ */
+static bool
+matching_restriction_variables(PlannerInfo *root, List *args, int varRelid)
+{
+	Node	   *left,
+			   *right;
+	Bitmapset  *varnos;
+
+	/* Fail if not a binary opclause (probably shouldn't happen) */
+	if (list_length(args) != 2)
+		return false;
+
+	left = (Node *) linitial(args);
+	right = (Node *) lsecond(args);
+
+	/* Look inside any binary-compatible relabeling */
+
+	if (IsA(left, RelabelType))
+		left = (Node *) ((RelabelType *) left)->arg;
+
+	if (IsA(right, RelabelType))
+		right = (Node *) ((RelabelType *) right)->arg;
+
+	/*
+	 * Check if it's safe to compare the two expressions. The expressions
+	 * must not contain any volatile expressions, and must belong to the
+	 * same relation.
+	 *
+	 * XXX We do a small trick - we validate just one expression, and then
+	 * check it's equal to the other side.
+	 */
+	if (contain_volatile_functions(left))
+		return false;
+
+	/* Check it belongs to a single relation. */
+	varnos = pull_varnos(root, left);
+
+	if (bms_num_members(varnos) != 1)
+		return false;
+
+	if ((varRelid != 0) && (!bms_is_member(varRelid, varnos)))
+		return false;
+
+	/* The two variables need to match */
+	return equal(left, right);
+}
+
 /*
  * get_join_variables
  *		Apply examine_variable() to each side of a join clause.
diff --git a/src/test/regress/expected/stats_ext.out b/src/test/regress/expected/stats_ext.out
index 009c7cfd848..fea3d18442c 100644
--- a/src/test/regress/expected/stats_ext.out
+++ b/src/test/regress/expected/stats_ext.out
@@ -1997,31 +1997,31 @@ SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE a = a');
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE a != a');
  estimated | actual 
 -----------+--------
-      4975 |      0
+         1 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE a <= a');
  estimated | actual 
 -----------+--------
-      1667 |   5000
+      5000 |   5000
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE a < a');
  estimated | actual 
 -----------+--------
-      1667 |      0
+         1 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE a > a');
  estimated | actual 
 -----------+--------
-      1667 |      0
+         1 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE a >= a');
  estimated | actual 
 -----------+--------
-      1667 |   5000
+      5000 |   5000
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE a = c');
@@ -2216,31 +2216,31 @@ SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE a = a');
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE a != a');
  estimated | actual 
 -----------+--------
-      4975 |      0
+         1 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE a <= a');
  estimated | actual 
 -----------+--------
-      1667 |   5000
+      5000 |   5000
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE a < a');
  estimated | actual 
 -----------+--------
-      1667 |      0
+         1 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE a > a');
  estimated | actual 
 -----------+--------
-      1667 |      0
+         1 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE a >= a');
  estimated | actual 
 -----------+--------
-      1667 |   5000
+      5000 |   5000
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE a = c');
@@ -2697,13 +2697,13 @@ SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b = d');
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b = d AND a > a');
  estimated | actual 
 -----------+--------
-         8 |      0
+         1 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b = d AND a < a');
  estimated | actual 
 -----------+--------
-         8 |      0
+         1 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b = d AND a = a');
@@ -2715,31 +2715,31 @@ SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b = d AND a =
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b = d AND NOT a = a');
  estimated | actual 
 -----------+--------
-        25 |      0
+         1 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b = d OR a > a');
  estimated | actual 
 -----------+--------
-      1683 |   2500
+        25 |   2500
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b = d OR a < a');
  estimated | actual 
 -----------+--------
-      1683 |   2500
+        25 |   2500
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b = d OR a = a');
  estimated | actual 
 -----------+--------
-        50 |   2500
+        25 |   2500
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b = d OR NOT a = a');
  estimated | actual 
 -----------+--------
-      4975 |   2500
+        25 |   2500
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b != d');
@@ -2751,13 +2751,13 @@ SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b != d');
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b != d AND a > a');
  estimated | actual 
 -----------+--------
-      1658 |      0
+         1 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b != d AND a < a');
  estimated | actual 
 -----------+--------
-      1658 |      0
+         1 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b != d AND a = a');
@@ -2769,19 +2769,19 @@ SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b != d AND a =
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b != d AND NOT a = a');
  estimated | actual 
 -----------+--------
-      4950 |      0
+         1 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b != d OR a > a');
  estimated | actual 
 -----------+--------
-      4983 |      0
+      4975 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b != d OR a < a');
  estimated | actual 
 -----------+--------
-      4983 |      0
+      4975 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b != d OR a = a');
@@ -2793,7 +2793,7 @@ SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b != d OR a =
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b != d OR NOT a = a');
  estimated | actual 
 -----------+--------
-      5000 |      0
+      4975 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b < d');
@@ -2805,13 +2805,13 @@ SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b < d');
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b < d AND a > a');
  estimated | actual 
 -----------+--------
-       556 |      0
+         1 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b < d AND a < a');
  estimated | actual 
 -----------+--------
-       556 |      0
+         1 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b < d AND a = a');
@@ -2823,31 +2823,31 @@ SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b < d AND a =
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b < d AND NOT a = a');
  estimated | actual 
 -----------+--------
-      1658 |      0
+         1 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b < d OR a > a');
  estimated | actual 
 -----------+--------
-      2778 |      0
+      1667 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b < d OR a < a');
  estimated | actual 
 -----------+--------
-      2778 |      0
+      1667 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b < d OR a = a');
  estimated | actual 
 -----------+--------
-      1683 |      0
+      1667 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b < d OR NOT a = a');
  estimated | actual 
 -----------+--------
-      4983 |      0
+      1667 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b <= d');
@@ -2859,13 +2859,13 @@ SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b <= d');
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b <= d AND a > a');
  estimated | actual 
 -----------+--------
-       556 |      0
+         1 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b <= d AND a < a');
  estimated | actual 
 -----------+--------
-       556 |      0
+         1 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b <= d AND a = a');
@@ -2877,31 +2877,31 @@ SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b <= d AND a =
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b <= d AND NOT a = a');
  estimated | actual 
 -----------+--------
-      1658 |      0
+         1 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b <= d OR a > a');
  estimated | actual 
 -----------+--------
-      2778 |   2500
+      1667 |   2500
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b <= d OR a < a');
  estimated | actual 
 -----------+--------
-      2778 |   2500
+      1667 |   2500
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b <= d OR a = a');
  estimated | actual 
 -----------+--------
-      1683 |   2500
+      1667 |   2500
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b <= d OR NOT a = a');
  estimated | actual 
 -----------+--------
-      4983 |   2500
+      1667 |   2500
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b > d');
@@ -2913,13 +2913,13 @@ SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b > d');
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b > d AND a > a');
  estimated | actual 
 -----------+--------
-       556 |      0
+         1 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b > d AND a < a');
  estimated | actual 
 -----------+--------
-       556 |      0
+         1 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b > d AND a = a');
@@ -2931,31 +2931,31 @@ SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b > d AND a =
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b > d AND NOT a = a');
  estimated | actual 
 -----------+--------
-      1658 |      0
+         1 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b > d OR a > a');
  estimated | actual 
 -----------+--------
-      2778 |      0
+      1667 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b > d OR a < a');
  estimated | actual 
 -----------+--------
-      2778 |      0
+      1667 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b > d OR a = a');
  estimated | actual 
 -----------+--------
-      1683 |      0
+      1667 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b > d OR NOT a = a');
  estimated | actual 
 -----------+--------
-      4983 |      0
+      1667 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b >= d');
@@ -2967,13 +2967,13 @@ SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b >= d');
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b >= d AND a > a');
  estimated | actual 
 -----------+--------
-       556 |      0
+         1 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b >= d AND a < a');
  estimated | actual 
 -----------+--------
-       556 |      0
+         1 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b >= d AND a = a');
@@ -2985,43 +2985,43 @@ SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b >= d AND a =
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b >= d AND NOT a = a');
  estimated | actual 
 -----------+--------
-      1658 |      0
+         1 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b >= d OR a > a');
  estimated | actual 
 -----------+--------
-      2778 |   2500
+      1667 |   2500
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b >= d OR a < a');
  estimated | actual 
 -----------+--------
-      2778 |   2500
+      1667 |   2500
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b >= d OR a = a');
  estimated | actual 
 -----------+--------
-      1683 |   2500
+      1667 |   2500
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b >= d OR NOT a = a');
  estimated | actual 
 -----------+--------
-      4983 |   2500
+      1667 |   2500
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b = d AND c > c');
  estimated | actual 
 -----------+--------
-         8 |      0
+         1 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b = d AND c < c');
  estimated | actual 
 -----------+--------
-         8 |      0
+         1 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b = d AND c = c');
@@ -3033,31 +3033,31 @@ SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b = d AND c =
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b = d AND NOT c = c');
  estimated | actual 
 -----------+--------
-        25 |      0
+         1 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b = d OR c > c');
  estimated | actual 
 -----------+--------
-      1683 |   2500
+        25 |   2500
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b = d OR c < c');
  estimated | actual 
 -----------+--------
-      1683 |   2500
+        25 |   2500
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b = d OR c = c');
  estimated | actual 
 -----------+--------
-        50 |   2500
+      2513 |   2500
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b = d OR NOT c = c');
  estimated | actual 
 -----------+--------
-      4975 |   2500
+        25 |   2500
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b != d');
@@ -3069,13 +3069,13 @@ SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b != d');
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b != d AND c > c');
  estimated | actual 
 -----------+--------
-      1658 |      0
+         1 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b != d AND c < c');
  estimated | actual 
 -----------+--------
-      1658 |      0
+         1 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b != d AND c = c');
@@ -3087,31 +3087,31 @@ SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b != d AND c =
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b != d AND NOT c = c');
  estimated | actual 
 -----------+--------
-      4950 |      0
+         1 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b != d OR c > c');
  estimated | actual 
 -----------+--------
-      4983 |      0
+      4975 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b != d OR c < c');
  estimated | actual 
 -----------+--------
-      4983 |      0
+      4975 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b != d OR c = c');
  estimated | actual 
 -----------+--------
-      4975 |   2500
+      4988 |   2500
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b != d OR NOT c = c');
  estimated | actual 
 -----------+--------
-      5000 |      0
+      4975 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b < d');
@@ -3123,13 +3123,13 @@ SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b < d');
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b < d AND c > c');
  estimated | actual 
 -----------+--------
-       556 |      0
+         1 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b < d AND c < c');
  estimated | actual 
 -----------+--------
-       556 |      0
+         1 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b < d AND c = c');
@@ -3141,31 +3141,31 @@ SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b < d AND c =
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b < d AND NOT c = c');
  estimated | actual 
 -----------+--------
-      1658 |      0
+         1 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b < d OR c > c');
  estimated | actual 
 -----------+--------
-      2778 |      0
+      1667 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b < d OR c < c');
  estimated | actual 
 -----------+--------
-      2778 |      0
+      1667 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b < d OR c = c');
  estimated | actual 
 -----------+--------
-      1683 |   2500
+      3333 |   2500
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b < d OR NOT c = c');
  estimated | actual 
 -----------+--------
-      4983 |      0
+      1667 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b <= d');
@@ -3177,13 +3177,13 @@ SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b <= d');
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b <= d AND c > c');
  estimated | actual 
 -----------+--------
-       556 |      0
+         1 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b <= d AND c < c');
  estimated | actual 
 -----------+--------
-       556 |      0
+         1 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b <= d AND c = c');
@@ -3195,31 +3195,31 @@ SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b <= d AND c =
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b <= d AND NOT c = c');
  estimated | actual 
 -----------+--------
-      1658 |      0
+         1 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b <= d OR c > c');
  estimated | actual 
 -----------+--------
-      2778 |   2500
+      1667 |   2500
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b <= d OR c < c');
  estimated | actual 
 -----------+--------
-      2778 |   2500
+      1667 |   2500
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b <= d OR c = c');
  estimated | actual 
 -----------+--------
-      1683 |   2500
+      3333 |   2500
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b <= d OR NOT c = c');
  estimated | actual 
 -----------+--------
-      4983 |   2500
+      1667 |   2500
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b > d');
@@ -3231,13 +3231,13 @@ SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b > d');
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b > d AND c > c');
  estimated | actual 
 -----------+--------
-       556 |      0
+         1 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b > d AND c < c');
  estimated | actual 
 -----------+--------
-       556 |      0
+         1 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b > d AND c = c');
@@ -3249,31 +3249,31 @@ SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b > d AND c =
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b > d AND NOT c = c');
  estimated | actual 
 -----------+--------
-      1658 |      0
+         1 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b > d OR c > c');
  estimated | actual 
 -----------+--------
-      2778 |      0
+      1667 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b > d OR c < c');
  estimated | actual 
 -----------+--------
-      2778 |      0
+      1667 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b > d OR c = c');
  estimated | actual 
 -----------+--------
-      1683 |   2500
+      3333 |   2500
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b > d OR NOT c = c');
  estimated | actual 
 -----------+--------
-      4983 |      0
+      1667 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b >= d');
@@ -3285,13 +3285,13 @@ SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b >= d');
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b >= d AND c > c');
  estimated | actual 
 -----------+--------
-       556 |      0
+         1 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b >= d AND c < c');
  estimated | actual 
 -----------+--------
-       556 |      0
+         1 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b >= d AND c = c');
@@ -3303,31 +3303,31 @@ SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b >= d AND c =
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b >= d AND NOT c = c');
  estimated | actual 
 -----------+--------
-      1658 |      0
+         1 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b >= d OR c > c');
  estimated | actual 
 -----------+--------
-      2778 |   2500
+      1667 |   2500
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b >= d OR c < c');
  estimated | actual 
 -----------+--------
-      2778 |   2500
+      1667 |   2500
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b >= d OR c = c');
  estimated | actual 
 -----------+--------
-      1683 |   2500
+      3333 |   2500
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b >= d OR NOT c = c');
  estimated | actual 
 -----------+--------
-      4983 |   2500
+      1667 |   2500
 (1 row)
 
 -- create statistics
@@ -3372,13 +3372,13 @@ SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b = d');
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b = d AND a > a');
  estimated | actual 
 -----------+--------
-         8 |      0
+         1 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b = d AND a < a');
  estimated | actual 
 -----------+--------
-         8 |      0
+         1 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b = d AND a = a');
@@ -3390,31 +3390,31 @@ SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b = d AND a =
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b = d AND NOT a = a');
  estimated | actual 
 -----------+--------
-        25 |      0
+         1 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b = d OR a > a');
  estimated | actual 
 -----------+--------
-      1683 |   2500
+        25 |   2500
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b = d OR a < a');
  estimated | actual 
 -----------+--------
-      1683 |   2500
+        25 |   2500
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b = d OR a = a');
  estimated | actual 
 -----------+--------
-        50 |   2500
+        25 |   2500
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b = d OR NOT a = a');
  estimated | actual 
 -----------+--------
-      4975 |   2500
+        25 |   2500
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b != d');
@@ -3426,13 +3426,13 @@ SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b != d');
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b != d AND a > a');
  estimated | actual 
 -----------+--------
-      1658 |      0
+         1 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b != d AND a < a');
  estimated | actual 
 -----------+--------
-      1658 |      0
+         1 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b != d AND a = a');
@@ -3444,19 +3444,19 @@ SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b != d AND a =
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b != d AND NOT a = a');
  estimated | actual 
 -----------+--------
-      4950 |      0
+         1 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b != d OR a > a');
  estimated | actual 
 -----------+--------
-      4983 |      0
+      4975 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b != d OR a < a');
  estimated | actual 
 -----------+--------
-      4983 |      0
+      4975 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b != d OR a = a');
@@ -3468,7 +3468,7 @@ SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b != d OR a =
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b != d OR NOT a = a');
  estimated | actual 
 -----------+--------
-      5000 |      0
+      4975 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b < d');
@@ -3480,13 +3480,13 @@ SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b < d');
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b < d AND a > a');
  estimated | actual 
 -----------+--------
-       556 |      0
+         1 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b < d AND a < a');
  estimated | actual 
 -----------+--------
-       556 |      0
+         1 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b < d AND a = a');
@@ -3498,31 +3498,31 @@ SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b < d AND a =
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b < d AND NOT a = a');
  estimated | actual 
 -----------+--------
-      1658 |      0
+         1 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b < d OR a > a');
  estimated | actual 
 -----------+--------
-      2778 |      0
+      1667 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b < d OR a < a');
  estimated | actual 
 -----------+--------
-      2778 |      0
+      1667 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b < d OR a = a');
  estimated | actual 
 -----------+--------
-      1683 |      0
+      1667 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b < d OR NOT a = a');
  estimated | actual 
 -----------+--------
-      4983 |      0
+      1667 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b <= d');
@@ -3534,13 +3534,13 @@ SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b <= d');
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b <= d AND a > a');
  estimated | actual 
 -----------+--------
-       556 |      0
+         1 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b <= d AND a < a');
  estimated | actual 
 -----------+--------
-       556 |      0
+         1 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b <= d AND a = a');
@@ -3552,31 +3552,31 @@ SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b <= d AND a =
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b <= d AND NOT a = a');
  estimated | actual 
 -----------+--------
-      1658 |      0
+         1 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b <= d OR a > a');
  estimated | actual 
 -----------+--------
-      2778 |   2500
+      1667 |   2500
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b <= d OR a < a');
  estimated | actual 
 -----------+--------
-      2778 |   2500
+      1667 |   2500
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b <= d OR a = a');
  estimated | actual 
 -----------+--------
-      1683 |   2500
+      1667 |   2500
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b <= d OR NOT a = a');
  estimated | actual 
 -----------+--------
-      4983 |   2500
+      1667 |   2500
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b > d');
@@ -3588,13 +3588,13 @@ SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b > d');
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b > d AND a > a');
  estimated | actual 
 -----------+--------
-       556 |      0
+         1 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b > d AND a < a');
  estimated | actual 
 -----------+--------
-       556 |      0
+         1 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b > d AND a = a');
@@ -3606,31 +3606,31 @@ SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b > d AND a =
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b > d AND NOT a = a');
  estimated | actual 
 -----------+--------
-      1658 |      0
+         1 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b > d OR a > a');
  estimated | actual 
 -----------+--------
-      2778 |      0
+      1667 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b > d OR a < a');
  estimated | actual 
 -----------+--------
-      2778 |      0
+      1667 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b > d OR a = a');
  estimated | actual 
 -----------+--------
-      1683 |      0
+      1667 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b > d OR NOT a = a');
  estimated | actual 
 -----------+--------
-      4983 |      0
+      1667 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b >= d');
@@ -3642,13 +3642,13 @@ SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b >= d');
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b >= d AND a > a');
  estimated | actual 
 -----------+--------
-       556 |      0
+         1 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b >= d AND a < a');
  estimated | actual 
 -----------+--------
-       556 |      0
+         1 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b >= d AND a = a');
@@ -3660,43 +3660,43 @@ SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b >= d AND a =
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b >= d AND NOT a = a');
  estimated | actual 
 -----------+--------
-      1658 |      0
+         1 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b >= d OR a > a');
  estimated | actual 
 -----------+--------
-      2778 |   2500
+      1667 |   2500
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b >= d OR a < a');
  estimated | actual 
 -----------+--------
-      2778 |   2500
+      1667 |   2500
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b >= d OR a = a');
  estimated | actual 
 -----------+--------
-      1683 |   2500
+      1667 |   2500
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b >= d OR NOT a = a');
  estimated | actual 
 -----------+--------
-      4983 |   2500
+      1667 |   2500
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b = d AND c > c');
  estimated | actual 
 -----------+--------
-         8 |      0
+         1 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b = d AND c < c');
  estimated | actual 
 -----------+--------
-         8 |      0
+         1 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b = d AND c = c');
@@ -3708,31 +3708,31 @@ SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b = d AND c =
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b = d AND NOT c = c');
  estimated | actual 
 -----------+--------
-        25 |      0
+         1 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b = d OR c > c');
  estimated | actual 
 -----------+--------
-      1683 |   2500
+        25 |   2500
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b = d OR c < c');
  estimated | actual 
 -----------+--------
-      1683 |   2500
+        25 |   2500
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b = d OR c = c');
  estimated | actual 
 -----------+--------
-        50 |   2500
+      2513 |   2500
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b = d OR NOT c = c');
  estimated | actual 
 -----------+--------
-      4975 |   2500
+        25 |   2500
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b != d');
@@ -3744,13 +3744,13 @@ SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b != d');
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b != d AND c > c');
  estimated | actual 
 -----------+--------
-      1658 |      0
+         1 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b != d AND c < c');
  estimated | actual 
 -----------+--------
-      1658 |      0
+         1 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b != d AND c = c');
@@ -3762,31 +3762,31 @@ SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b != d AND c =
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b != d AND NOT c = c');
  estimated | actual 
 -----------+--------
-      4950 |      0
+         1 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b != d OR c > c');
  estimated | actual 
 -----------+--------
-      4983 |      0
+      4975 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b != d OR c < c');
  estimated | actual 
 -----------+--------
-      4983 |      0
+      4975 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b != d OR c = c');
  estimated | actual 
 -----------+--------
-      4975 |   2500
+      4988 |   2500
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b != d OR NOT c = c');
  estimated | actual 
 -----------+--------
-      5000 |      0
+      4975 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b < d');
@@ -3798,13 +3798,13 @@ SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b < d');
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b < d AND c > c');
  estimated | actual 
 -----------+--------
-       556 |      0
+         1 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b < d AND c < c');
  estimated | actual 
 -----------+--------
-       556 |      0
+         1 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b < d AND c = c');
@@ -3816,31 +3816,31 @@ SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b < d AND c =
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b < d AND NOT c = c');
  estimated | actual 
 -----------+--------
-      1658 |      0
+         1 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b < d OR c > c');
  estimated | actual 
 -----------+--------
-      2778 |      0
+      1667 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b < d OR c < c');
  estimated | actual 
 -----------+--------
-      2778 |      0
+      1667 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b < d OR c = c');
  estimated | actual 
 -----------+--------
-      1683 |   2500
+      3333 |   2500
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b < d OR NOT c = c');
  estimated | actual 
 -----------+--------
-      4983 |      0
+      1667 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b <= d');
@@ -3852,13 +3852,13 @@ SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b <= d');
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b <= d AND c > c');
  estimated | actual 
 -----------+--------
-       556 |      0
+         1 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b <= d AND c < c');
  estimated | actual 
 -----------+--------
-       556 |      0
+         1 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b <= d AND c = c');
@@ -3870,31 +3870,31 @@ SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b <= d AND c =
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b <= d AND NOT c = c');
  estimated | actual 
 -----------+--------
-      1658 |      0
+         1 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b <= d OR c > c');
  estimated | actual 
 -----------+--------
-      2778 |   2500
+      1667 |   2500
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b <= d OR c < c');
  estimated | actual 
 -----------+--------
-      2778 |   2500
+      1667 |   2500
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b <= d OR c = c');
  estimated | actual 
 -----------+--------
-      1683 |   2500
+      3333 |   2500
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b <= d OR NOT c = c');
  estimated | actual 
 -----------+--------
-      4983 |   2500
+      1667 |   2500
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b > d');
@@ -3906,13 +3906,13 @@ SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b > d');
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b > d AND c > c');
  estimated | actual 
 -----------+--------
-       556 |      0
+         1 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b > d AND c < c');
  estimated | actual 
 -----------+--------
-       556 |      0
+         1 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b > d AND c = c');
@@ -3924,31 +3924,31 @@ SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b > d AND c =
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b > d AND NOT c = c');
  estimated | actual 
 -----------+--------
-      1658 |      0
+         1 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b > d OR c > c');
  estimated | actual 
 -----------+--------
-      2778 |      0
+      1667 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b > d OR c < c');
  estimated | actual 
 -----------+--------
-      2778 |      0
+      1667 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b > d OR c = c');
  estimated | actual 
 -----------+--------
-      1683 |   2500
+      3333 |   2500
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b > d OR NOT c = c');
  estimated | actual 
 -----------+--------
-      4983 |      0
+      1667 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b >= d');
@@ -3960,13 +3960,13 @@ SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b >= d');
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b >= d AND c > c');
  estimated | actual 
 -----------+--------
-       556 |      0
+         1 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b >= d AND c < c');
  estimated | actual 
 -----------+--------
-       556 |      0
+         1 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b >= d AND c = c');
@@ -3978,31 +3978,31 @@ SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b >= d AND c =
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b >= d AND NOT c = c');
  estimated | actual 
 -----------+--------
-      1658 |      0
+         1 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b >= d OR c > c');
  estimated | actual 
 -----------+--------
-      2778 |   2500
+      1667 |   2500
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b >= d OR c < c');
  estimated | actual 
 -----------+--------
-      2778 |   2500
+      1667 |   2500
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b >= d OR c = c');
  estimated | actual 
 -----------+--------
-      1683 |   2500
+      3333 |   2500
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists WHERE b >= d OR NOT c = c');
  estimated | actual 
 -----------+--------
-      4983 |   2500
+      1667 |   2500
 (1 row)
 
 -- mcv with pass-by-ref fixlen types, e.g. uuid
@@ -4102,7 +4102,7 @@ SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists_bool WHERE NOT a AND
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists_bool WHERE NOT a = a');
  estimated | actual 
 -----------+--------
-      9950 |      0
+         1 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists_bool WHERE NOT a != a');
@@ -4114,25 +4114,25 @@ SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists_bool WHERE NOT a !=
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists_bool WHERE NOT a < a');
  estimated | actual 
 -----------+--------
-      3333 |  10000
+     10000 |  10000
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists_bool WHERE NOT a <= a');
  estimated | actual 
 -----------+--------
-      3333 |      0
+         1 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists_bool WHERE NOT a > a');
  estimated | actual 
 -----------+--------
-      3333 |  10000
+     10000 |  10000
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists_bool WHERE NOT a >= a');
  estimated | actual 
 -----------+--------
-      3333 |      0
+         1 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists_bool WHERE NOT a = b');
@@ -4174,37 +4174,37 @@ SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists_bool WHERE NOT a >=
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists_bool WHERE NOT a = a AND b = b');
  estimated | actual 
 -----------+--------
-      9950 |      0
+         1 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists_bool WHERE NOT a != a AND b != b');
  estimated | actual 
 -----------+--------
-      9950 |      0
+         1 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists_bool WHERE NOT a < a AND b < b');
  estimated | actual 
 -----------+--------
-      1111 |      0
+         1 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists_bool WHERE NOT a <= a AND b <= b');
  estimated | actual 
 -----------+--------
-      1111 |      0
+         1 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists_bool WHERE NOT a > a AND b > b');
  estimated | actual 
 -----------+--------
-      1111 |      0
+         1 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists_bool WHERE NOT a >= a AND b >= b');
  estimated | actual 
 -----------+--------
-      1111 |      0
+         1 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists_bool WHERE NOT a = b AND b = c');
@@ -4252,31 +4252,31 @@ SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists_bool WHERE NOT a = c
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists_bool WHERE NOT a != c AND b != b');
  estimated | actual 
 -----------+--------
-        50 |      0
+         1 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists_bool WHERE NOT a < c AND b < b');
  estimated | actual 
 -----------+--------
-      1111 |      0
+         1 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists_bool WHERE NOT a <= a AND b <= c');
  estimated | actual 
 -----------+--------
-      1111 |      0
+         1 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists_bool WHERE NOT a > a AND b > c');
  estimated | actual 
 -----------+--------
-      1111 |   1250
+      3333 |   1250
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists_bool WHERE NOT a >= a AND b >= c');
  estimated | actual 
 -----------+--------
-      1111 |      0
+         1 |      0
 (1 row)
 
 CREATE STATISTICS mcv_lists_bool_stats (mcv) ON a, b, c
@@ -4309,7 +4309,7 @@ SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists_bool WHERE NOT a AND
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists_bool WHERE NOT a = a');
  estimated | actual 
 -----------+--------
-      9950 |      0
+         1 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists_bool WHERE NOT a != a');
@@ -4321,25 +4321,25 @@ SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists_bool WHERE NOT a !=
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists_bool WHERE NOT a < a');
  estimated | actual 
 -----------+--------
-      3333 |  10000
+     10000 |  10000
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists_bool WHERE NOT a <= a');
  estimated | actual 
 -----------+--------
-      3333 |      0
+         1 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists_bool WHERE NOT a > a');
  estimated | actual 
 -----------+--------
-      3333 |  10000
+     10000 |  10000
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists_bool WHERE NOT a >= a');
  estimated | actual 
 -----------+--------
-      3333 |      0
+         1 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists_bool WHERE NOT a = b');
@@ -4381,37 +4381,37 @@ SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists_bool WHERE NOT a >=
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists_bool WHERE NOT a = a AND b = b');
  estimated | actual 
 -----------+--------
-      9950 |      0
+         1 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists_bool WHERE NOT a != a AND b != b');
  estimated | actual 
 -----------+--------
-      9950 |      0
+         1 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists_bool WHERE NOT a < a AND b < b');
  estimated | actual 
 -----------+--------
-      1111 |      0
+         1 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists_bool WHERE NOT a <= a AND b <= b');
  estimated | actual 
 -----------+--------
-      1111 |      0
+         1 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists_bool WHERE NOT a > a AND b > b');
  estimated | actual 
 -----------+--------
-      1111 |      0
+         1 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists_bool WHERE NOT a >= a AND b >= b');
  estimated | actual 
 -----------+--------
-      1111 |      0
+         1 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists_bool WHERE NOT a = b AND b = c');
@@ -4459,31 +4459,31 @@ SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists_bool WHERE NOT a = c
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists_bool WHERE NOT a != c AND b != b');
  estimated | actual 
 -----------+--------
-        50 |      0
+         1 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists_bool WHERE NOT a < c AND b < b');
  estimated | actual 
 -----------+--------
-      1111 |      0
+         1 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists_bool WHERE NOT a <= a AND b <= c');
  estimated | actual 
 -----------+--------
-      1111 |      0
+         1 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists_bool WHERE NOT a > a AND b > c');
  estimated | actual 
 -----------+--------
-      1111 |   1250
+      3333 |   1250
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists_bool WHERE NOT a >= a AND b >= c');
  estimated | actual 
 -----------+--------
-      1111 |      0
+         1 |      0
 (1 row)
 
 -- mcv covering just a small fraction of data
@@ -4742,13 +4742,13 @@ SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists_multi WHERE a = a AN
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists_multi WHERE a != b AND c != c');
  estimated | actual 
 -----------+--------
-      4950 |      0
+         1 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists_multi WHERE a < a AND c > d');
  estimated | actual 
 -----------+--------
-       556 |      0
+         1 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists_multi WHERE a <= b AND d >= c');
@@ -4760,19 +4760,19 @@ SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists_multi WHERE a <= b A
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists_multi WHERE a = a OR c = d');
  estimated | actual 
 -----------+--------
-        50 |   5000
+      5000 |   5000
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists_multi WHERE a != b OR c != c');
  estimated | actual 
 -----------+--------
-      5000 |      0
+      4975 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists_multi WHERE a < a OR c > d');
  estimated | actual 
 -----------+--------
-      2778 |      0
+      1667 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists_multi WHERE a <= b OR d >= c');
@@ -4908,13 +4908,13 @@ SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists_multi WHERE a = a AN
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists_multi WHERE a != b AND c != c');
  estimated | actual 
 -----------+--------
-      4950 |      0
+         1 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists_multi WHERE a < a AND c > d');
  estimated | actual 
 -----------+--------
-       556 |      0
+         1 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists_multi WHERE a <= b AND d >= c');
@@ -4926,19 +4926,19 @@ SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists_multi WHERE a <= b A
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists_multi WHERE a = a OR c = d');
  estimated | actual 
 -----------+--------
-        50 |   5000
+      5000 |   5000
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists_multi WHERE a != b OR c != c');
  estimated | actual 
 -----------+--------
-      5000 |      0
+      4975 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists_multi WHERE a < a OR c > d');
  estimated | actual 
 -----------+--------
-      2778 |      0
+      1667 |      0
 (1 row)
 
 SELECT * FROM check_estimated_rows('SELECT * FROM mcv_lists_multi WHERE a <= b OR d >= c');
-- 
2.34.3

